﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using Infragistics;
using System.Windows.Data;
using System.Windows;

namespace TestProject
{
	public class ScheduleData
	{
		private ObservableCollection<ResourceItem> _resources;
		private ObservableCollection<ResourceCalendarItem> _calendars;
		private ObservableCollection<AppointmentItem> _appointments;

		public ScheduleData()
		{
			_resources = new ObservableCollection<ResourceItem>();
			_calendars = new ObservableCollection<ResourceCalendarItem>();
			_appointments = new ObservableCollection<AppointmentItem>();
		}

		public ObservableCollection<ResourceItem> Resources { get { return _resources; } }
		public ObservableCollection<ResourceCalendarItem> Calendars { get { return _calendars; } }
		public ObservableCollection<AppointmentItem> Appointments { get { return _appointments; } }

		internal TimeslotType GetTimeslotType(DateTime startTime, DateTime endTime, bool isWorkingHour)
		{
			if (!isWorkingHour)
				return TimeslotType.NonWorkingTime;

			DateRange lunchTime = new DateRange(startTime.Date.AddHours(12), startTime.Date.AddHours(13).AddSeconds(-1));
			DateRange timeslotRange = new DateRange(startTime, endTime.AddSeconds(-1));

			if (lunchTime.IntersectsWith(timeslotRange))
				return TimeslotType.LunchTime;

			return TimeslotType.WorkingTime;
		}

		#region TimeslotType

		/// <summary>
		/// TimeslotType Attached Dependency Property
		/// </summary>
		public static readonly DependencyProperty TimeslotTypeProperty =
			DependencyProperty.RegisterAttached("TimeslotType", typeof(TimeslotType), typeof(ScheduleData),
				new FrameworkPropertyMetadata((TimeslotType)TimeslotType.WorkingTime));

		public static TimeslotType GetTimeslotType(DependencyObject d)
		{
			return (TimeslotType)d.GetValue(TimeslotTypeProperty);
		}

		public static void SetTimeslotType(DependencyObject d, TimeslotType value)
		{
			d.SetValue(TimeslotTypeProperty, value);
		}

		#endregion //TimeslotType
	}

	public enum TimeslotType
	{
		NonWorkingTime,
		WorkingTime,
		LunchTime,
	}

	public class TimeslotTypeValueConverter : IMultiValueConverter
	{
		public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			if (values == null || values.Length != 4)
				return DependencyProperty.UnsetValue;

			if (values[0] is ScheduleData == false ||
				values[1] is DateTime == false ||
				values[2] is DateTime == false ||
				values[3] is Boolean == false)
				return DependencyProperty.UnsetValue;

			ScheduleData data = values[0] as ScheduleData;
			DateTime startTime = (DateTime)values[1];
			DateTime endTime = (DateTime)values[2];
			bool isWorkingHour = (bool)values[3];

			return data.GetTimeslotType(startTime, endTime, isWorkingHour);
		}

		public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
		{
			return new object[] { DependencyProperty.UnsetValue };
		}
	}
}
